@@ -11,6 +11,7 @@
 # under the License.
 
 from openstack.block_storage import _base_proxy
+from openstack.block_storage.v3 import availability_zone
 from openstack.block_storage.v3 import backup as _backup
 from openstack.block_storage.v3 import snapshot as _snapshot
 from openstack.block_storage.v3 import stats as _stats
@@ -175,6 +176,129 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._update(_type.Type, type, **attrs)
 
+    def update_type_extra_specs(self, type, **attrs):
+        """Update the extra_specs for a type
+
+        :param type: The value can be either the ID of a type or a
+                     :class:`~openstack.volume.v3.type.Type` instance.
+        :param dict attrs: The extra_spec attributes to update on the
+                           type represented by ``value``.
+
+        :returns: A dict containing updated extra_specs
+
+        """
+        res = self._get_resource(_type.Type, type)
+        extra_specs = res.set_extra_specs(self, **attrs)
+        result = _type.Type.existing(id=res.id, extra_specs=extra_specs)
+        return result
+
+    def delete_type_extra_specs(self, type, keys):
+        """Delete the extra_specs for a type
+
+        Note: This method will do a HTTP DELETE request for every key in keys.
+
+        :param type: The value can be either the ID of a type or a
+                     :class:`~openstack.volume.v3.type.Type` instance.
+        :param keys: The keys to delete
+
+        :returns: ``None``
+        """
+        res = self._get_resource(_type.Type, type)
+        return res.delete_extra_specs(self, keys)
+
+    def get_type_encryption(self, volume_type_id):
+        """Get the encryption details of a volume type
+
+        :param volume_type_id: The value can be the ID of a type or a
+                               :class:`~openstack.volume.v3.type.Type`
+                               instance.
+
+        :returns: One :class:`~openstack.volume.v3.type.TypeEncryption`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        volume_type = self._get_resource(_type.Type, volume_type_id)
+
+        return self._get(_type.TypeEncryption,
+                         volume_type_id=volume_type.id,
+                         requires_id=False)
+
+    def create_type_encryption(self, volume_type, **attrs):
+        """Create new type encryption from attributes
+
+        :param volume_type: The value can be the ID of a type or a
+                            :class:`~openstack.volume.v3.type.Type`
+                            instance.
+
+        :param dict attrs: Keyword arguments which will be used to create
+                           a :class:`~openstack.volume.v3.type.TypeEncryption`,
+                           comprised of the properties on the TypeEncryption
+                           class.
+
+        :returns: The results of type encryption creation
+        :rtype: :class:`~openstack.volume.v3.type.TypeEncryption`
+        """
+        volume_type = self._get_resource(_type.Type, volume_type)
+
+        return self._create(_type.TypeEncryption,
+                            volume_type_id=volume_type.id, **attrs)
+
+    def delete_type_encryption(self, encryption=None,
+                               volume_type=None, ignore_missing=True):
+        """Delete type encryption attributes
+
+        :param encryption: The value can be None or a
+                           :class:`~openstack.volume.v3.type.TypeEncryption`
+                           instance.  If encryption_id is None then
+                           volume_type_id must be specified.
+
+        :param volume_type: The value can be the ID of a type or a
+                            :class:`~openstack.volume.v3.type.Type`
+                            instance.  Required if encryption_id is None.
+
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the type does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent type.
+
+        :returns: ``None``
+        """
+
+        if volume_type:
+            volume_type = self._get_resource(_type.Type, volume_type)
+            encryption = self._get(_type.TypeEncryption,
+                                   volume_type=volume_type.id,
+                                   requires_id=False)
+
+        self._delete(_type.TypeEncryption, encryption,
+                     ignore_missing=ignore_missing)
+
+    def update_type_encryption(self, encryption=None,
+                               volume_type=None, **attrs):
+        """Update a type
+        :param encryption: The value can be None or a
+                           :class:`~openstack.volume.v3.type.TypeEncryption`
+                           instance.  If encryption_id is None then
+                           volume_type_id must be specified.
+
+        :param volume_type: The value can be the ID of a type or a
+                            :class:`~openstack.volume.v3.type.Type`
+                            instance.  Required if encryption_id is None.
+        :param dict attrs: The attributes to update on the type encryption.
+
+        :returns: The updated type encryption
+        :rtype: :class:`~openstack.volume.v3.type.TypeEncryption`
+        """
+
+        if volume_type:
+            volume_type = self._get_resource(_type.Type, volume_type)
+            encryption = self._get(_type.TypeEncryption,
+                                   volume_type=volume_type.id,
+                                   requires_id=False)
+
+        return self._update(_type.TypeEncryption, encryption, **attrs)
+
     def get_volume(self, volume):
         """Get a single volume
 
@@ -290,10 +414,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :returns: A generator of backup objects.
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         base_path = '/backups/detail' if details else None
         return self._list(_backup.Backup, base_path=base_path, **query)
 
@@ -307,10 +427,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: Backup instance
         :rtype: :class:`~openstack.block_storage.v3.backup.Backup`
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         return self._get(_backup.Backup, backup)
 
     def find_backup(self, name_or_id, ignore_missing=True, **attrs):
@@ -325,10 +441,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
                  when no resource can be found.
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         return self._find(_backup.Backup, name_or_id,
                           ignore_missing=ignore_missing)
 
@@ -342,10 +454,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: The results of Backup creation
         :rtype: :class:`~openstack.block_storage.v3.backup.Backup`
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         return self._create(_backup.Backup, **attrs)
 
     def delete_backup(self, backup, ignore_missing=True):
@@ -361,10 +469,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :returns: ``None``
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         self._delete(_backup.Backup, backup,
                      ignore_missing=ignore_missing)
 
@@ -379,13 +483,19 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: Updated backup instance
         :rtype: :class:`~openstack.block_storage.v3.backup.Backup`
         """
-        if not self._connection.has_service('object-store'):
-            raise exceptions.SDKException(
-                'Object-store service is required for block-store backups'
-            )
         backup = self._get_resource(_backup.Backup, backup)
         return backup.restore(self, volume_id=volume_id, name=name)
 
+    def availability_zones(self):
+        """Return a generator of availability zones
+
+        :returns: A generator of availability zone
+        :rtype: :class:`~openstack.block_storage.v3.availability_zone.\
+                        AvailabilityZone`
+        """
+
+        return self._list(availability_zone.AvailabilityZone)
+
     def wait_for_status(self, res, status='ACTIVE', failures=None,
                         interval=2, wait=120):
         """Wait for a resource to be in a particular status.
@@ -437,29 +547,26 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
     def _service_cleanup(self, dry_run=True, client_status_queue=None,
                          identified_resources=None,
                          filters=None, resource_evaluation_fn=None):
-        if self._connection.has_service('object-store'):
-            # Volume backups require object-store to be available, even for
-            # listing
-            backups = []
-            for obj in self.backups(details=False):
-                need_delete = self._service_cleanup_del_res(
-                    self.delete_backup,
-                    obj,
-                    dry_run=dry_run,
-                    client_status_queue=client_status_queue,
-                    identified_resources=identified_resources,
-                    filters=filters,
-                    resource_evaluation_fn=resource_evaluation_fn)
-                if not dry_run and need_delete:
-                    backups.append(obj)
+        backups = []
+        for obj in self.backups(details=False):
+            need_delete = self._service_cleanup_del_res(
+                self.delete_backup,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+            if not dry_run and need_delete:
+                backups.append(obj)
 
-            # Before deleting snapshots need to wait for backups to be deleted
-            for obj in backups:
-                try:
-                    self.wait_for_delete(obj)
-                except exceptions.SDKException:
-                    # Well, did our best, still try further
-                    pass
+        # Before deleting snapshots need to wait for backups to be deleted
+        for obj in backups:
+            try:
+                self.wait_for_delete(obj)
+            except exceptions.SDKException:
+                # Well, did our best, still try further
+                pass
 
         snapshots = []
         for obj in self.snapshots(details=False):
