@@ -179,6 +179,13 @@ Additional information about the services can be found in the
 import warnings
 import weakref
 
+try:
+    # For python 3.8 and later
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    # For everyone else
+    import importlib_metadata
+
 import concurrent.futures
 import keystoneauth1.exceptions
 import requestsexceptions
@@ -201,6 +208,7 @@ from openstack.cloud import _network_common
 from openstack.cloud import _object_store
 from openstack.cloud import _orchestration
 from openstack.cloud import _security_group
+from openstack.cloud import _shared_file_system
 from openstack import config as _config
 from openstack.config import cloud_region
 from openstack import exceptions
@@ -264,7 +272,8 @@ class Connection(
     _network_common.NetworkCommonCloudMixin,
     _object_store.ObjectStoreCloudMixin,
     _orchestration.OrchestrationCloudMixin,
-    _security_group.SecurityGroupCloudMixin
+    _security_group.SecurityGroupCloudMixin,
+    _shared_file_system.SharedFileSystemCloudMixin,
 ):
 
     def __init__(self, cloud=None, config=None, session=None,
@@ -408,6 +417,7 @@ class Connection(
         _object_store.ObjectStoreCloudMixin.__init__(self)
         _orchestration.OrchestrationCloudMixin.__init__(self)
         _security_group.SecurityGroupCloudMixin.__init__(self)
+        _shared_file_system.SharedFileSystemCloudMixin.__init__(self)
 
         # Allow vendors to provide hooks. They will normally only receive a
         # connection object and a responsible to register additional services
@@ -423,15 +433,17 @@ class Connection(
                     (package_name, function) = vendor_hook.rsplit(':')
 
                     if package_name and function:
-                        import pkg_resources
-                        ep = pkg_resources.EntryPoint(
-                            'vendor_hook', package_name, attrs=(function,))
-                        hook = ep.resolve()
+                        ep = importlib_metadata.EntryPoint(
+                            name='vendor_hook',
+                            value=vendor_hook,
+                            group='vendor_hook',
+                        )
+                        hook = ep.load()
                         hook(self)
                 except ValueError:
                     self.log.warning('Hook should be in the entrypoint '
                                      'module:attribute format')
-            except (ImportError, TypeError) as e:
+            except (ImportError, TypeError, AttributeError) as e:
                 self.log.warning('Configured hook %s cannot be executed: %s',
                                  vendor_hook, e)
 
