@@ -14,10 +14,10 @@ from openstack.identity.v2 import extension as _extension
 from openstack.identity.v2 import role as _role
 from openstack.identity.v2 import tenant as _tenant
 from openstack.identity.v2 import user as _user
-from openstack import proxy2 as proxy
+from openstack import proxy
 
 
-class Proxy(proxy.BaseProxy):
+class Proxy(proxy.Proxy):
 
     def extensions(self):
         """Retrieve a generator of extensions
@@ -25,7 +25,7 @@ class Proxy(proxy.BaseProxy):
         :returns: A generator of extension instances.
         :rtype: :class:`~openstack.identity.v2.extension.Extension`
         """
-        return self._list(_extension.Extension, paginated=False)
+        return self._list(_extension.Extension)
 
     def get_extension(self, extension):
         """Get a single extension
@@ -96,13 +96,13 @@ class Proxy(proxy.BaseProxy):
     def roles(self, **query):
         """Retrieve a generator of roles
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of role instances.
         :rtype: :class:`~openstack.identity.v2.role.Role`
         """
-        return self._list(_role.Role, paginated=False, **query)
+        return self._list(_role.Role, **query)
 
     def update_role(self, role, **attrs):
         """Update a role
@@ -173,13 +173,13 @@ class Proxy(proxy.BaseProxy):
     def tenants(self, **query):
         """Retrieve a generator of tenants
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of tenant instances.
         :rtype: :class:`~openstack.identity.v2.tenant.Tenant`
         """
-        return self._list(_tenant.Tenant, paginated=True, **query)
+        return self._list(_tenant.Tenant, **query)
 
     def update_tenant(self, tenant, **attrs):
         """Update a tenant
@@ -250,13 +250,13 @@ class Proxy(proxy.BaseProxy):
     def users(self, **query):
         """Retrieve a generator of users
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of user instances.
         :rtype: :class:`~openstack.identity.v2.user.User`
         """
-        return self._list(_user.User, paginated=False, **query)
+        return self._list(_user.User, **query)
 
     def update_user(self, user, **attrs):
         """Update a user
