@@ -574,6 +574,13 @@ class TestCase(base.TestCase):
         return dict(method='GET', uri="https://accelerator.example.com/",
                     text=open(discovery_fixture, 'r').read())
 
+    def get_manila_discovery_mock_dict(self):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, "shared-file-system.json")
+        return dict(method='GET',
+                    uri="https://shared-file-system.example.com/",
+                    text=open(discovery_fixture, 'r').read())
+
     def use_glance(
             self, image_version_json='image-version.json',
             image_discovery_url='https://image.example.com/'):
@@ -630,6 +637,15 @@ class TestCase(base.TestCase):
         self.__do_register_uris([
             self.get_cyborg_discovery_mock_dict()])
 
+    def use_manila(self):
+        # NOTE(gouthamr): This method is only meant to be used in "setUp"
+        # where the ordering of the url being registered is tightly controlled
+        # if the functionality of .use_manila is meant to be used during an
+        # actual test case, use .get_manila_discovery_mock and apply to the
+        # right location in the mock_uris when calling .register_uris
+        self.__do_register_uris([
+            self.get_manila_discovery_mock_dict()])
+
     def register_uris(self, uri_mock_list=None):
         """Mock a list of URIs and responses via requests mock.
 
