@@ -34,10 +34,9 @@ class TestQoSPolicy(base.TestCase):
         self.assertEqual('policy', sot.resource_key)
         self.assertEqual('policies', sot.resources_key)
         self.assertEqual('/qos/policies', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -46,8 +45,8 @@ class TestQoSPolicy(base.TestCase):
         self.assertEqual(EXAMPLE['id'], sot.id)
         self.assertEqual(EXAMPLE['description'], sot.description)
         self.assertEqual(EXAMPLE['name'], sot.name)
-        self.assertTrue(sot.is_shared)
         self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
+        self.assertEqual(EXAMPLE['tenant_id'], sot.tenant_id)
         self.assertEqual(EXAMPLE['rules'], sot.rules)
         self.assertEqual(EXAMPLE['is_default'], sot.is_default)
         self.assertEqual(EXAMPLE['tags'], sot.tags)
